import 'reflect-metadata';
import ApolloClient from 'apollo-client';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { resolvers, schema as typeDefs} from '@workbench-stack/extension-module-browser';
import {
    CompletionProviderDefinitionDocument,
    DocumentFormattingProviderDefinitionDocument,
    ReferenceProviderDefinitionDocument,
    DocumentSymbolDefinitionDocument,
    DefinitionDefinitionDocument,
    DocumentHighLightProviderDefinitionDocument,
    RenameDefinitionDocument,
    HoverProviderDefinitionDocument,
    UpdateModelDocument,
} from '@workbench-stack/components';

const defaultSchema = `
type Query {
    dummy: Int
}
type Mutation {
    dummy: Int
}
`;

const cache = new InMemoryCache();



export const client = new ApolloClient({
    typeDefs: defaultSchema.concat(typeDefs),
    resolvers,
    queryDeduplication: true,
    // dataIdFromObject: (result) => modules.getDataIdFromObject(result),
    cache,
});


export const updateModelMutate = async(data) => {
    try {
        const res = await client.mutate({
            mutation: UpdateModelDocument,
            variables: data.variables,
            context: data.context,
        });
        return res;
    } catch (err) {
        console.error('err', err);
    }
};

export const completionMutate = async(data) => {
    try {
        await client.mutate({
            mutation: CompletionProviderDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const formattingMutate = async(data) => {
    try {
        await client.mutate({
            mutation: DocumentFormattingProviderDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const referenceMutate = async(data) => {
    try {
        await client.mutate({
            mutation: ReferenceProviderDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const symbolMutate = async(data) => {
    try {
        await client.mutate({
            mutation: DocumentSymbolDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const hoverMutate = async(data) => {
    try {
        await client.mutate({
            mutation: HoverProviderDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const highlightMutate = async(data) => {
    try {
        await client.mutate({
            mutation: DocumentHighLightProviderDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const renameEditsMutate = async(data) => {
    try {
        await client.mutate({
            mutation: RenameDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};

export const definitionMutate = async(data) => {
    try {
        await client.mutate({
            mutation: DefinitionDefinitionDocument,
            variables: data.variables,
            context: data.context,
        });
    } catch (err) {
        console.error('err', err);
    }
};
