
import * as React from 'react';
import { createSlotFill } from '@workbench-stack/components';
import { EDITOR_SLOT_PROVIDER } from '../constants';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { IEditorRegistry, IEditorInput, CacheTypenames } from '@workbench-stack/core';
import { ClientExtensions } from '../constants';



export type IEditorComponentProps = { input: IEditorInput, visibleEditorId: string };


type IEditorSlotProps = { input: IEditorInput };

export const { Fill: EditorContributionFill, Slot: EditorSlot }  = createSlotFill<IEditorComponentProps>(EDITOR_SLOT_PROVIDER);

export const Editor: React.FC<IEditorSlotProps> = (props) => {
    const registry = Registry.as<IEditorRegistry>(ClientExtensions.Editors);
    const typename = props.input ? props.input.__typename : CacheTypenames.ResourceEditorInput;
    const editorDescriptor = registry.getEditor({ __typename: typename});

    return React.useMemo(() => {
        return (
            editorDescriptor ? <EditorSlot fillProps={{...props, visibleEditorId: editorDescriptor.id }} /> : null
        );
    }, [props.input.resource]);
};




