/* tslint:disable */
import * as group from "./editor-group-helper";
import "jest";
import { IEditorInput, EditorOpenPositioning, CloseDirection } from "@workbench-stack/core";

describe("Workbench editor groups", () => {

    it("Clone Group", function () { });

    it("contains() with diff editor support", function () { });

    it("One Editor", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBe(true);

        editorOpenedFiles = group.closeEditor(input1, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();

        // Active && Preview
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: false });
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBeFalsy();
        editorOpenedFiles = group.closeEditor(input2, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();

        // Nonactive && Pinned => gets active because its first editor
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: false, pinned: true });
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBe(true);

        editorOpenedFiles = group.closeEditor(input3, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();


        // Nonactive && Unpinned => gets active because its first editor
        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBeFalsy();

        editorOpenedFiles = group.closeEditor(input4, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();

    });

    it('Multiple Editors - Pinned and Active', function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        // Pinned and Active
        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true });

        expect(editorOpenedFiles.length).toBe(3);
        expect(editorOpenedFiles[0].label).toBe('file1');
        expect(editorOpenedFiles[0].isActive).toBe(false);
        expect(editorOpenedFiles[0].isPinned).toBe(true);
        expect(editorOpenedFiles[1].label).toBe('file2');
        expect(editorOpenedFiles[1].isActive).toBe(false);
        expect(editorOpenedFiles[1].isPinned).toBe(true);
        expect(editorOpenedFiles[2].isActive).toBe(true);
        expect(editorOpenedFiles[2].isPinned).toBe(true);

        const mru = group.getEditors();
        expect(mru[0].label).toBe('file3');
        expect(mru[1].label).toBe('file2');
        expect(mru[2].label).toBe('file1');
        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it('Multiple Editors - Preview(preopen) editor moves to the side of the active one', function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: false });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true });
        expect(editorOpenedFiles[2].label).toBe('file3');

        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: false }); // this should cause the preview editor to move after input3
        expect(editorOpenedFiles[2].label).toBe('file4');

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it('Multiple Editos - Pinned and Active (DEFAULT_OPEN_EDITOR_DIRECTION = Direction.LEFT)', function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true, editorOpenPositioning: EditorOpenPositioning.LEFT });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true, editorOpenPositioning: EditorOpenPositioning.LEFT });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true, editorOpenPositioning: EditorOpenPositioning.LEFT });

        expect(editorOpenedFiles[0].label).toBe('file3');
        expect(editorOpenedFiles[1].label).toBe('file2');
        expect(editorOpenedFiles[2].label).toBe('file1');

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it("Multiple Editors - Pinned and Not Active", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { pinned: true });

        expect(editorOpenedFiles.length).toBe(3);
        expect(group.getEditors().length).toBe(3);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBe(true);
        expect(editorOpenedFiles[1].isActive).toBeFalsy();
        expect(editorOpenedFiles[1].isPinned).toBe(true);
        expect(editorOpenedFiles[2].isActive).toBeFalsy();
        expect(editorOpenedFiles[2].isPinned).toBe(true);

        const mru = group.getEditors();
        expect(mru[0].label).toBe('file1');
        expect(mru[1].label).toBe('file2');
        expect(mru[2].label).toBe('file3');

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it("Multiple Editors - Preview gets overwritten", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        // Non active, preview
        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles); // becomes active, preview
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles); // overwrites preview
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles); // overwrites preview

        expect(editorOpenedFiles.length).toBe(1);
        expect(group.getEditors().length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].isPinned).toBeFalsy();

        const mru = group.getEditors();
        expect(mru[0].label).toBe('file3');
        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it("Multiple Editors - set active", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: false });

        expect(editorOpenedFiles.length).toBe(3);

        editorOpenedFiles = group.activateEditor(input3, editorOpenedFiles);
        expect(editorOpenedFiles[2].isActive).toBe(true);

        editorOpenedFiles = group.activateEditor(input1, editorOpenedFiles);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[1].isActive).toBeFalsy();
        expect(editorOpenedFiles[2].isActive).toBeFalsy();

        let mru = group.getEditors();
        expect(mru[0].label).toBe('file1');
        expect(mru[1].label).toBe('file3');
        expect(mru[2].label).toBe('file2');

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        expect(editorOpenedFiles.length).toBeFalsy();
    });

    it("Multiple Editors - pin and unpin", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: false });

        expect(editorOpenedFiles.length).toBe(3);
        editorOpenedFiles = group.pin(input3, editorOpenedFiles);


        expect(editorOpenedFiles[2].isPinned).toBe(true);
        expect(editorOpenedFiles[2].isActive).toBe(true);

        editorOpenedFiles = group.unpin(editorOpenedFiles[0], editorOpenedFiles);

        expect(editorOpenedFiles[0].isPinned).toBe(false);
        expect(editorOpenedFiles[0].isActive).toBe(false);

        editorOpenedFiles = group.unpin(editorOpenedFiles[1], editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(2);
        expect(editorOpenedFiles[0].isPinned).toBe(false);

        editorOpenedFiles = group.unpin(editorOpenedFiles[1], editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isPinned).toBe(false);

    });

    it("Multiple Editors - closing picks next from MRU list", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        const input5: IEditorInput = {
            editorId: 1,
            resource: "a://5",
            label: "file5",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { active: true, pinned: true });

        expect(editorOpenedFiles.length).toBe(5);
        expect(group.getEditors()[0].label).toBe('file5');
        expect(editorOpenedFiles[4].isActive).toBe(true);

        // close input 5
        editorOpenedFiles = group.closeEditor(input5, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(4);
        expect(group.getEditors()[0].label).toBe('file4');
        expect(group.getEditors()[0].isActive).toBe(true);


        // close input 4
        editorOpenedFiles = group.closeEditor(input4, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(3);
        expect(group.getEditors()[0].isActive).toBe(true);
        expect(group.getEditors()[0].label).toBe('file3');

        // close input 1
        editorOpenedFiles = group.closeEditor(input1, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(2);
        expect(group.getEditors()[0].isActive).toBe(true);
        expect(group.getEditors()[0].label).toBe('file3');

        // activate input 2
        editorOpenedFiles = group.activateEditor(input2, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(2);
        expect(editorOpenedFiles[1].label).toBe('file3')
        expect(group.getEditors()[0].isActive).toBe(true);
        expect(group.getEditors()[0].label).toBe('file2');

        // close input 2
        editorOpenedFiles = group.closeEditor(input2, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].isActive).toBe(true);
        expect(editorOpenedFiles[0].label).toBe('file3');

        // close 3
        editorOpenedFiles = group.closeEditor(input3, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(0);
        console.log('--', editorOpenedFiles);
    });

    it("Multiple Editors - closing picks next to the right", function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        const input5: IEditorInput = {
            editorId: 1,
            resource: "a://5",
            label: "file5",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });

        expect(group.activeEditor(editorOpenedFiles).label).toBe(input5.label);
        expect(group.getEditors()[0].label).toBe(input5.label);
        expect(editorOpenedFiles.length).toBe(5);

        editorOpenedFiles = group.closeEditor(input5, editorOpenedFiles, { focusRecentEditorAfterClose: false });
        expect(group.getEditors()[0].label).toBe(input4.label);
        expect(editorOpenedFiles.length).toBe(4);

        editorOpenedFiles = group.activateEditor(input1, editorOpenedFiles);
        editorOpenedFiles = group.closeEditor(input1, editorOpenedFiles, { focusRecentEditorAfterClose: false });

        expect(group.activeEditor(editorOpenedFiles).label).toBe(input2.label);
        expect(editorOpenedFiles.length).toBe(3);


        editorOpenedFiles = group.activateEditor(input3, editorOpenedFiles);
        editorOpenedFiles = group.closeEditor(input3, editorOpenedFiles, { focusRecentEditorAfterClose: false });

        expect(group.activeEditor(editorOpenedFiles).label).toBe(input4.label);
        expect(editorOpenedFiles.length).toBe(2);

        editorOpenedFiles = group.closeEditor(input4, editorOpenedFiles, { focusRecentEditorAfterClose: false });

        editorOpenedFiles = group.activateEditor(input2, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(1);

        editorOpenedFiles = group.closeEditor(input2, editorOpenedFiles, { focusRecentEditorAfterClose: false });

        expect(editorOpenedFiles.length).toBe(0);
    });

    it('Multiple Editors - move editor', function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        const input5: IEditorInput = {
            editorId: 1,
            resource: "a://5",
            label: "file5",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { pinned: true, active: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { pinned: true, active: true });

        editorOpenedFiles = group.moveEditor(input1, editorOpenedFiles, 1);

        expect(editorOpenedFiles[0].label).toBe(input2.label);
        expect(editorOpenedFiles[1].label).toBe(input1.label);

        editorOpenedFiles = group.activateEditor(input1, editorOpenedFiles);
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { pinned: true, active: true });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { pinned: true, active: true });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { pinned: true, active: true });

        editorOpenedFiles = group.moveEditor(input4, editorOpenedFiles, 0);

        expect(editorOpenedFiles[0].label).toBe(input4.label);
        expect(editorOpenedFiles[1].label).toBe(input2.label);
        expect(editorOpenedFiles[2].label).toBe(input1.label);
        expect(editorOpenedFiles[3].label).toBe(input3.label);
        expect(editorOpenedFiles[4].label).toBe(input5.label);

        editorOpenedFiles = group.moveEditor(input4, editorOpenedFiles, 3);
        editorOpenedFiles = group.moveEditor(input2, editorOpenedFiles, 1);

        expect(editorOpenedFiles[0].label).toBe(input1.label);
        expect(editorOpenedFiles[1].label).toBe(input2.label);
        expect(editorOpenedFiles[2].label).toBe(input3.label);
        expect(editorOpenedFiles[3].label).toBe(input4.label);
        expect(editorOpenedFiles[4].label).toBe(input5.label);
    })

    it('Multiple Editors - move editor accross groups', function () {
        let editorOpenedFiles1: IEditorInput[] = [];
        let editorOpenedFiles2: IEditorInput[] = [];

        const g1_input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const g1_input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const g2_input1: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles1 = group.openEditor(g1_input1, editorOpenedFiles1, { pinned: true, active: true });
        editorOpenedFiles1 = group.openEditor(g1_input2, editorOpenedFiles1, { pinned: true, active: true });
        editorOpenedFiles2 = group.openEditor(g2_input1, editorOpenedFiles2, { pinned: true, active: true });

        // A move across groups is a close in the one group and an open in the other group at a specific index
        editorOpenedFiles2 = group.closeEditor(g2_input1, editorOpenedFiles2);
        editorOpenedFiles1 = group.openEditor(g2_input1, editorOpenedFiles1, { active: true, pinned: true, index: 1 });

        expect(editorOpenedFiles1.length).toBe(3);
        expect(editorOpenedFiles1[0].label).toBe(g1_input1.label);
        expect(editorOpenedFiles1[1].label).toBe(g2_input1.label);
        expect(editorOpenedFiles1[2].label).toBe(g1_input2.label);
    });

    it('Multiple Editors - Pinned & Non Active', function () {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles);
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(group.preopenEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(editorOpenedFiles[0].label).toBe(input1.label);
        expect(editorOpenedFiles.length).toBe(1);

        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { pinned: true, active: false });
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(group.preopenEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(editorOpenedFiles[0].label).toBe(input1.label);
        expect(editorOpenedFiles[1].label).toBe(input2.label);
        expect(editorOpenedFiles.length).toBe(2);

        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { pinned: true, active: false });
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(group.preopenEditor(editorOpenedFiles).label).toBe(input1.label);
        expect(editorOpenedFiles[0].label).toBe(input1.label);
        expect(editorOpenedFiles[1].label).toBe(input3.label);
        expect(editorOpenedFiles[2].label).toBe(input2.label);
        expect(editorOpenedFiles[0].isPinned).toBeFalsy();
        expect(editorOpenedFiles[1].isPinned).toBe(true);
        expect(editorOpenedFiles[2].isPinned).toBe(true);
        expect(editorOpenedFiles.length).toBe(3);
    });


    it('Multiple Editors - Close Others, Close Left, Close Right', function() {
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }
        const input4: IEditorInput = {
            editorId: 1,
            resource: "a://4",
            label: "file4",
        }
        const input5: IEditorInput = {
            editorId: 1,
            resource: "a://5",
            label: "file5",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });

        // Close Others
        editorOpenedFiles = group.closeEditors(group.activeEditor(editorOpenedFiles)!, editorOpenedFiles);
        expect(editorOpenedFiles.length).toBe(1);
        expect(editorOpenedFiles[0].label).toBe(input5.label);

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.activateEditor(input3, editorOpenedFiles);

        // Close left
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input3.label);
        editorOpenedFiles = group.closeEditors(group.activeEditor(editorOpenedFiles)!, editorOpenedFiles,  CloseDirection.LEFT);
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input3.label);
        expect(editorOpenedFiles.length).toBe(3);
        expect(editorOpenedFiles[0].label).toBe(input3.label);
        expect(editorOpenedFiles[1].label).toBe(input4.label);
        expect(editorOpenedFiles[2].label).toBe(input5.label);

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input3, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input4, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.openEditor(input5, editorOpenedFiles, { active: true, pinned: true, focusRecentEditorAfterClose: false });
        editorOpenedFiles = group.activateEditor(input3, editorOpenedFiles);

        // Close Right
        expect(group.activeEditor(editorOpenedFiles).label).toBe(input3.label);
        editorOpenedFiles = group.closeEditors(group.activeEditor(editorOpenedFiles)!, editorOpenedFiles, CloseDirection.RIGHT);
        expect(editorOpenedFiles.length).toBe(3);
        expect(editorOpenedFiles[0].label).toBe(input1.label);
        expect(editorOpenedFiles[1].label).toBe(input2.label);
        expect(editorOpenedFiles[2].label).toBe(input3.label);

        editorOpenedFiles = group.closeAllEditors(editorOpenedFiles);
    });

    it('Multiple Editors - real user example', function (){

    });

    it('Preopen(Preview) tab does not have a stable position (https://github.com/Microsoft/vscode/issues/8245)', function (){
        let editorOpenedFiles: IEditorInput[] = [];

        const input1: IEditorInput = {
            editorId: 1,
            resource: "a://1",
            label: "file1",
        };
        const input2: IEditorInput = {
            editorId: 1,
            resource: "a://2",
            label: "file2",
        }
        const input3: IEditorInput = {
            editorId: 1,
            resource: "a://3",
            label: "file3",
        }

        editorOpenedFiles = group.openEditor(input1, editorOpenedFiles, { active: true, pinned: true });
        editorOpenedFiles = group.openEditor(input2, editorOpenedFiles, { active: true });
        editorOpenedFiles = group.activateEditor(input1, editorOpenedFiles);

        editorOpenedFiles =  group.openEditor(input3, editorOpenedFiles, { active: true });
        expect(group.indexOf(input3, editorOpenedFiles)).toBe(1);
    });

    it('Multiple Editors - Editor Emits Dirty and Label Changed', function () {
        
    })
});
