

import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';


export interface GitUriParams {
    path: string;
    ref: string;
    submoduleOf?: string;
}

export function fromGitUri(uri: URI): GitUriParams {
    return JSON.parse(uri.query);
}

export interface GitUriOptions {
    replaceFileExtension?: boolean;
    submoduleOf?: string;
}


// As a mitigation for extensions like ESLint showing warnings and errors
// for git URIs, let's change the file extension of these uris to .git,
// when `replaceFileExtension` is true.
export function toGitUri(uri: URI, ref: string, options: GitUriOptions = {}): URI {
    const params: GitUriParams = {
        path: uri.fsPath,
        ref,
    };

    if (options.submoduleOf) {
        params.submoduleOf = options.submoduleOf;
    }

    let path = uri.path;

    if (options.replaceFileExtension) {
        path = `${path}.git`;
    } else if (options.submoduleOf) {
        path = `${path}.diff`;
    }

    return uri.with({
        scheme: 'git',
        path,
        query: JSON.stringify(params),
    });
}

